function [ClData,CdData,CmData,alphaData,MachData] = getAeroData() 
%--------------------------------------------------------------------------
% getAeroData() 
% Calls the vehicle aerodynamics solver for a range of Mach and angle of attack
% numbers to get a 3D grid of lift, drag, and moment coefficients.
%--------------------------------------------------------------------------    

%Load in vehicle parameters
vehicle = [{"Ariane"},{21},{45*pi/180},{11.5},{3.77},{4.5},{6.9},{19.63},{0},{2},{0},{28.62},{5.114},{1.918},{1.28}];
%vehicle = [{"Shuttle"},{23.79},{45*pi/180},{17.507},{3.5},{6.5},{3.7},{11.59},{0},{3},{0.65},{19},{8.87},{2.24},{1}];

%Calculate all geometric parameters
[bw,bMAC,cMAC,cRoot,AR,taper,sweepLE,sweepTE,sweep25,Sw,tBar,ctMax,phiTE,airfoilAero,A2,B2,A3,xU,xL,Ucoeffs,Lcoeffs] = wingGeometry(vehicle);
[bf,cf,bFlapI,bFlapO] = flapGeometry(vehicle);
[db,ds,sigma,lb,ln,lf,la,Sb,Ss,Sp,Vb,Xp,Xcg,noseType] = bodyGeometry(vehicle,cRoot);

%Load into wing, body, flap structures to be input into aerodynamic solver
wing0 = {bw,bMAC,cMAC,cRoot,AR,taper,sweepLE,sweepTE,sweep25,Sw,tBar,ctMax,phiTE,airfoilAero,A2,B2,A3,xU,xL,Ucoeffs,Lcoeffs};
body0 = {db,ds,sigma,lb,ln,lf,la,Sb,Ss,Sp,Vb,Xp,Xcg,noseType};
flap = {bf,cf,bFlapI,bFlapO};

%Calculate the critical Mach number
Mcr = MachCr(tBar,AR,taper,sweepLE);

index1 = 0;
index2 = 0;

MachRange = [0:0.05:1.5,2:0.5:6,7:1:10];
alphaRange = -90*pi/180:3*pi/180:90*pi/180;

%Reynolds number, adiabatic index
Re = 10^6;
k = 1.4;

%Preallocate matrix side
ClData = zeros(length(MachRange),length(alphaRange));
CmData = zeros(length(MachRange),length(alphaRange));
CdData = zeros(length(MachRange),length(alphaRange));

%Call the solver for each M, aoa value
for M = MachRange
    delta = 0;
    index1 = 1+index1;
    index2 = 0;
    for alphaV = alphaRange
        index2 = 1+index2;
        [Cl,Cm,Cd,~,~] = vehicleAero(M,Re,k,alphaV,wing0,body0,flap,delta,Mcr);
        
        %Store the coefficient for each M, aoa        
        ClData(index1,index2) = Cl;
        CmData(index1,index2) = Cm;
        CdData(index1,index2) = Cd;
    end
end 

ClDeltaData = zeros(length(MachRange),1);
CmDeltaData = zeros(length(MachRange),1);
index1 = 0;

%Get the control derivatives, they vary negligibly with aoa        
for M = MachRange
     delta = 7*pi/180;
     alphaV = 10*pi/180;
     index1 = 1+index1;

     [~,~,~,ClDelta,CmDelta] = vehicleAero(M,Re,k,alphaV,wing0,body0,flap,delta,Mcr);

     ClDeltaData(index1) = ClDelta;
     CmDeltaData(index1) = CmDelta;
end 


[alphaData,MachData] = meshgrid(alphaRange,MachRange);
figure
surf(MachData,180/pi*alphaData,ClData)
figure
surf(MachData,180/pi*alphaData,CmData)
figure
surf(MachData,180/pi*alphaData,CdData)
figure
plot(MachRange,ClDeltaData)
figure
plot(MachRange,CmDeltaData)

[MachData2, alphaData2, ClData2] = prepareSurfaceData(MachData, alphaData, ClData);
[MachData2, alphaData2, CmData2] = prepareSurfaceData(MachData, alphaData, CmData);
[MachData2, alphaData2, CdData2] = prepareSurfaceData(MachData, alphaData, CdData);

exportClData = [MachData2, alphaData2, ClData2];
exportCmData = [MachData2, alphaData2, CmData2];
exportCdData = [MachData2, alphaData2, CdData2];

exportClDeltaData = [MachRange;ClDeltaData']';
exportCmDeltaData = [MachRange;CmDeltaData']';


%% Notes
%Wing area Sw is based on exposed area of wing
%Flaps located always inboard of tip, no taper
%Flap hinge line sweep is same angle as wing trailing edge sweep
%Flap deflection does not change total chord