function [Cl,Cm,Cd,ClDelta,CmDelta] = vehicleAero(M,Re,k,alphaV,wing0,body0,flap,delta,Mcr)
%--------------------------------------------------------------------------
% vehicleAero(M,Re,k,alphaV,wing0,body0,flap,delta,Mcr)
% Solves the lift, drag, moment coefficients for the inputted Mach number,
% angle of attack, vehicle geometry.
%--------------------------------------------------------------------------    
    alpha = alphaV;
    wing = wing0;
    body = body0;

    %Load back in the geometric properties of the vehicle
    [bw,bMAC,cMAC,cRoot,AR,taper,sweepLE,sweepTE,sweep25,Sw,tBar,ctMax,phiTE,airfoilAero,A2,B2,A3,xU,xL,Ucoeffs,Lcoeffs] = deal(wing{:});
    [bf,cf,bFlapI,bFlapO] = deal(flap{:});
    [db,ds,sigma,lb,ln,lf,la,Sb,Ss,Sp,Vb,Xp,Xcg,noseType]  = deal(body{:});

    %Calculate Reynolds number    
    ReW = Re*cMAC;
    ReB = Re*lb; 
    ReWSub = Mcr/M*ReW;
    ReBSub = Mcr/M*ReB;

    %Find the reference area and reference length
    Sref = Ss;
    lref = lb;
    
    %SOLVE IN THE SUBSONIC REGIME
    if Mcr >= M   
        %VEHICLE LIFT
        [ClW,ClAlpha,~,clAlpha] = SubClW(M,alpha,AR,taper,sweepLE,airfoilAero);
        ClB = SubClB(M,alpha,lb,db,Sb);
        [Kwb,Kbw,Kn] = KwbFactors(M,Mcr,ClAlpha,db,bw,la,cRoot,AR,taper,sweepLE,Sb,Sw);
        ClV = (Sb/Sref)*ClB+(Sw/Sref)*(Kn+Kwb+Kbw)*(ClW);

        %VEHICLE DRAG        
        CdfW = FunCdf(M,ReW,cMAC,Sw,Sref);
        CdpW = SubCdpW(CdfW,M,AR,taper,sweepLE,tBar,ctMax);
        CdiW = SubCdiW(ClW,0,ClAlpha,M,AR,taper,sweepLE);
        CdfB = FunCdf(M,ReB,lb,Ss,Sref);
        CdpB = SubCdpB(CdfB,db,lb);
        CdbB = SubCdbB(CdfB,CdpB);
        CdW = CdfW+CdpW+CdiW;
        CdB = CdfB+CdpB+CdbB;
        CdV = (CdfW+CdpW+CdiW)*(Sw/Sref)+(CdfB+CdpB)*(Ss/Sref)+CdbB*(Sb/Sref);

        %VEHICLE MOMENT        
        CmACW = FunCmW(AR,sweep25,tBar,airfoilAero);
        XacW = SubXacW(AR,taper,ln,lf,cRoot,bMAC,cMAC,sweepLE,airfoilAero);
        CmCGB = SubCmB(M,alpha,lb,db,Ss,Sp,Vb,Xp,Xcg);
        XacV = SubXacV(XacW,ClAlpha,M,ln,lf,db,bw,cRoot,AR,taper,sweepLE,sweep25,Sw,Kwb,Kbw,noseType);
        CmV = Sw/(Sref)*(CmACW+ClW*(Xcg-XacV)/lref)+CmCGB*Sp/Sref;

        %CONTROL DERIVATIVES        
        [ClF,ClDelta] = SubClF(ClAlpha,clAlpha,Re,AR,tBar,cMAC,cf,delta,bFlapI,bFlapO,phiTE);    
        CmCGF = SubCmF(ClF,bw,AR,taper,sweep25,tBar,cMAC,cf,bFlapI,bFlapO); 

        if delta == 0
            CmDelta = 0;
        else
            CmDelta = CmCGF/delta;
        end
    end  

    %SOLVE IN THE TRANSONIC REGIME
    if M > Mcr && 1.2 > M    
        %%%%% LOAD INTERPOLATION VALUES %%%%%
        [ClWSub,ClAlphaSub,alpha0Sub,clAlphaSub] = SubClW(Mcr,alpha,AR,taper,sweepLE,airfoilAero);
        [ClWSpr,ClAlphaSpr,alpha0Spr] = SprClW(1.2,k,alpha,AR,taper,sweepLE,A2);
        ClBSub = SubClB(Mcr,alpha,lb,db,Sb);
        ClBSpr = SprClB(1.2,alpha,lb,db,Sb);
        CdfBSub = FunCdf(M,ReBSub,lb,Ss,Sref);
        CdiWSub = SubCdiW(ClWSub,0,ClAlphaSub,Mcr,AR,taper,sweepLE);
        CdpBSub = SubCdpB(CdfBSub,db,lb);
        CdbBSub = SubCdbB(CdfBSub,CdpBSub);
        CdwWSpr = SprCdwW(1.2,k,alpha,AR,taper,sweepLE,A2,B2,A3);
        CmBSub = SubCmB(Mcr,alpha,lb,db,Ss,Sp,Vb,Xp,Xcg);
        CmBSpr = SprCmB(1.2,alpha,lb,Ss,Sp,Vb,Xcg,Xp);
        [ClFSub,~] = SubClF(ClAlphaSub,clAlphaSub,Re,AR,tBar,cMAC,cf,delta,bFlapI,bFlapO,phiTE); 
        [ClFSpr,~] = SprClF(1.2,k,phiTE,sweepTE,delta,Sw,bf,cf);
        CmCGFSub = SubCmF(ClFSub,bw,AR,taper,sweep25,tBar,cMAC,cf,bFlapI,bFlapO);
        CmCGFSpr = SprCmF(1.2,k,ln,lf,phiTE,bw,bf,bFlapI,sweepTE,cMAC,cRoot,cf,delta,Sw,Xcg);    
        %%%%% LOAD INTERPOLATION VALUES %%%%%

        %VEHICLE LIFT        
        [ClW,ClAlpha,~] = TranClW(ClWSub,ClWSpr,ClAlphaSub,ClAlphaSpr,alpha0Sub,alpha0Spr,M,Mcr,tBar);
        ClB = TranClB(ClBSub,ClBSpr,M,Mcr);
        [Kwb,Kbw,Kn] = KwbFactors(M,Mcr,ClAlpha,db,bw,la,cRoot,AR,taper,sweepLE,Sb,Sw);   
        ClV = (Sb/Sref)*ClB+(Sw/Sref)*(Kn+Kwb+Kbw)*(ClW);

        %VEHICLE DRAG        
        CdfW = FunCdf(M,ReW,cMAC,Sw,Sref);
        CdiW = TranCdiW(ClW,0,CdiWSub,M,Mcr,AR,taper,sweepLE,tBar);
        CdwW = TranCdwW(CdwWSpr,M,Mcr,tBar,AR,taper,sweepLE);
        CdfB = FunCdf(M,ReB,lb,Ss,Sref);
        CdpB = TranCdpB(CdpBSub,Mcr,M);
        CdbB = TranCdbB(CdbBSub,M);
        CdwB = FunCdwB(M,ln,db,sigma,noseType);
        CdW = CdfW+CdwW+CdiW;
        CdB = CdfB+CdpB+CdbB+CdwB;    
        CdV = (CdfW+CdiW+CdwW)*(Sw/Sref)+(CdfB+CdpB+CdwB)*(Ss/Sref)+CdbB*(Sb/Sref);

        %VEHICLE MOMENT        
        CmACW = FunCmW(AR,sweep25,tBar,airfoilAero);
        XacW = TranXacW(M,ln,lf,cRoot,sweepLE,AR,taper,tBar);
        CmCGB = TranCmB(CmBSub,CmBSpr,M,Mcr);
        XacVSub = SubXacV(XacW,ClAlpha,Mcr,ln,lf,db,bw,cRoot,AR,taper,sweepLE,sweep25,Sw,Kwb,Kbw,noseType);
        XacVSpr = SprXacV(XacW,ClAlpha,1.2,ln,lf,la,db,cRoot,AR,taper,sweepLE,Sw,Kwb,Kbw,noseType);
        XacV = TranXacV(XacVSub,XacVSpr,M,Mcr);
        CmV = Sw/(Sref)*(CmACW+ClW*(Xcg-XacV)/lref)+CmCGB*Sp/Sref;

        %CONTROL DERIVATIVES        
        [~, ClDelta] = TranClF(ClFSub,ClFSpr,M,Mcr,delta);    
        CmCGF = TranCmF(CmCGFSub,CmCGFSpr,M,Mcr);
        if delta == 0
            CmDelta = 0;
        else
            CmDelta = CmCGF/delta;
        end
    end  

    %SOLVE IN THE SUPERSONIC REGIME    
    if M >= 1.2 && 5 >= M
        %VEHICLE LIFT                
        [ClW,ClAlpha,~] = SprClW(M,k,alpha,AR,taper,sweepLE,A2);
        ClB = SprClB(M,alpha,lb,db,Sb);
        [Kwb,Kbw,Kn] = KwbFactors(M,Mcr,ClAlpha,db,bw,la,cRoot,AR,taper,sweepLE,Sb,Sw);    
        ClV = (Sb/Sref)*ClB+(Sw/Sref)*(Kn+Kwb+Kbw)*ClW;

        %VEHICLE LIFT        
        CdfW = FunCdf(M,ReW,cMAC,Sw,Sref);
        CdiW = SprCdiW(ClW,0,alpha);
        CdwW = SprCdwW(M,k,alpha,AR,taper,sweepLE,A2,B2,A3);
        CdsucW = SprCdsucW(ClW,0,M,AR,taper,sweepLE);
        CdfB = FunCdf(M,ReB,lb,Ss,Sref);
        CdbluntB = SprCdbluntB(ln,db,ds,M);
        CdbB = SprCdbB(M);
        CdwB = FunCdwB(M,ln,db,sigma,noseType);
        CdW = CdiW+CdwW+CdsucW+CdfW;
        CdB = CdbluntB+CdbB+CdwB+CdfB;        
        CdV = (CdiW+CdwW+CdsucW+CdfW)*(Sw/Sref)+(CdbluntB+CdwB+CdfB)*(Ss/Sref)+CdbB*(Sb/Sref);

        %VEHICLE MOMENT
        CmACW = FunCmW(AR,sweep25,tBar,airfoilAero);
        XacW = SprXacW(M,ln,lf,cRoot,sweepLE,taper,AR);
        CmCGB = SprCmB(M,alpha,lb,Ss,Sp,Vb,Xcg,Xp);
        XacV = SprXacV(XacW,ClAlpha,M,ln,lf,la,db,cRoot,AR,taper,sweepLE,Sw,Kwb,Kbw,noseType);
        CmV = Sw/(Sref)*(CmACW+ClW*(Xcg-XacV)/lref)+CmCGB*Sp/Sref;

        %CONTROL DERIVATIVES            
        [~,ClDelta] = SprClF(M,k,phiTE,sweepTE,delta,Sw,bf,cf);
        CmCGF = SprCmF(M,k,ln,lf,phiTE,bw,bf,bFlapI,sweepTE,cMAC,cRoot,cf,delta,Sw,Xcg);
        if delta == 0
            CmDelta = 0;
        else
            CmDelta = CmCGF/delta;
        end
    end  

    %SOLVE IN THE HYPERSONIC REGIME    
    if M > 5
        %VEHICLE LIFT
        [ClW,~] = HypClW(k,alpha,xU,xL,Ucoeffs,Lcoeffs);        
        [~,ClAlpha,~] = SprClW(M,k,alpha,AR,taper,sweepLE,A2); 
        [Kwb,Kbw,~] = KwbFactors(5,Mcr,ClAlpha,db,bw,la,cRoot,AR,taper,sweepLE,Sb,Sw);       
        ClB = HypClB(ln,db,ds,alpha,noseType);
        ClV = (Sb/Sref)*ClB+(Sw/Sref)*ClW;

        %VEHICLE DRAG        
        CdiW = HypCdiW(k,alpha,xU,xL,Ucoeffs,Lcoeffs);
        CdpW = HypCdpW(k,alpha,xU,xL,Ucoeffs,Lcoeffs);
        CdfW = FunCdf(M,ReW,cMAC,Sw,Sref);
        CdpB = HypCdpB(sigma,ln,ds,noseType);
        CdbB = HypCdbB(M); 
        CdfB = FunCdf(M,ReB,lb,Ss,Sref);
        CdW = CdiW+CdpW+CdfW;
        CdB = CdpB+CdbB+CdfB; 
        CdV = (CdiW+CdpW+CdfW)*(Sw/Sref)+(CdpB+CdfB)*(Ss/Sref)+CdbB*(Sb/Sref);

        %VEHICLE MOMENT        
        CmCGW = HypCmW(ClW,k,alpha,ln,lf,Xcg,xU,xL,Ucoeffs,Lcoeffs);
        CmCGB = HypCmB(ClB,alpha,lb,ln,db,ds,sigma,Xcg,noseType); 
        CmV = cMAC*Sw/(lref*Sref)*CmCGW+lb*Sp/(lref*Sref)*CmCGB;

        %CONTROL DERIVATIVES            
        [~,ClDelta] = SprClF(M,k,phiTE,sweepTE,delta,Sw,bf,cf);
        CmCGF = SprCmF(M,k,ln,lf,phiTE,bw,bf,bFlapI,sweepTE,cMAC,cRoot,cf,delta,Sw,Xcg);
        if delta == 0
            CmDelta = 0;
        else
            CmDelta = CmCGF/delta;
        end
    end
    
    %IF THE VEHICLE IS IN THE LINEAR ANGLE OF ATTACK RANGE 
    if alpha >= -15*pi/180 && alpha <= 15*pi/180
        %Results are output from above
        Cl = ClV;
        Cd = CdV;
        Cm = CmV;
    %IF THE VEHICLE IS IN THE NON-LINEAR ANGLE OF ATTACK RANGE         
    else
        %Low angle of attack estimates parsed in
        CdB0 = CdB*(Ss/Sref)-CdbB*(Ss/Sref)+CdbB*(Sb/Sref);
        CdW0 = CdW*(Sw/Sref);

        %Coefficients are instead replaced by the high angle of attack calculations 
        Cd = CdHighAlpha(CdB0,CdW0,M,alpha,lb,db,Sw,Sref);
        [Cl,ClW,ClB,ClWBAlpha,ClBWAlpha,ClWBDelta,ClBWDelta] = ClHighAlpha(CdB0,ClAlpha,M,Mcr,alpha,bw,AR,taper,sweepLE,lb,db,Sw,Sref,delta,Kwb,Kbw);
        [Cm,~,~] = CmHighAlpha(ClW,ClWBAlpha,ClBWAlpha,ClWBDelta,ClBWDelta,M,alpha,cRoot,bw,AR,taper,sweepLE,lb,ln,lf,db,Vb,Sref,Sw,Xcg,Xp);
    end
end

