function [ClW,ClAlpha,alpha0] = SprClW(M,k,alpha,AR,taper,sweepLE,A2)
%--------------------------------------------------------------------------
% SprClW(M,k,alpha,AR,taper,sweepLE,A2)
% SUPERSONIC WING LIFT COEFFICIENT
% Calculates the supersonic low angle of attack lift coeffient for the
% vehicle wing.
%--------------------------------------------------------------------------   
    beta = sqrt(M^2-1);
    %Supersonic factors from Busemann coefficients.
    C1 = 1/beta;
    C2 = ((M^2-2)^2+k*M^4)/(4*(M^2-1)^2);   
    alpha0 = -(C2/(2*C1))*(A2);
    
    ClW = findCl(alpha,alpha0,M,AR,taper,sweepLE,C1);

    %As the lift curve slope is found by dividing the lift coefficent by
    %the angle of attack, a relation that holds for symmetric aircraft in
    %the small alpha region, care must be made to avoid the singularity
    %when alpha = 0. An angle of attack slightly above and below zero are
    %used instead to calculate ClAlpha by finding the mean between them.
    alphaU = alpha0+0.5*pi/180;
    alphaL = alpha0-0.5*pi/180;
    ClU = findCl(alphaU,alpha0,M,AR,taper,sweepLE,C1);
    ClL = findCl(alphaL,alpha0,M,AR,taper,sweepLE,C1);
    ClAlphaU = ClU/(alphaU-alpha0);
    ClAlphaL = ClL/(alphaL-alpha0);
    
    ClAlpha = 0.5*(ClAlphaU+ClAlphaL);
end

function Cl = findCl(alpha,alpha0,M,AR,taper,sweepLE,C1)
    beta = sqrt(M^2-1);
    %Epsilon is the wing sweep measured from the longtidual axis
    epsilon = pi/2-sweepLE;
    
    %Cl0 is the wing lift coefficent without considering profile drag or
    %span effects. Clprofile is the additional effect due to the profile.
    %Clspan is the additional effect due to the span.
    Cl0 = 4*C1*alpha;
    Clprofile = -4*C1*alpha0;
    
    %mu is the angle of the Mach cone radiating from the leading edge of
    %the wing
    mu = asin(1/M);
    
    if tan(sweepLE) == (4/AR)*((1-taper)/((1+taper))) 
        %If the wing type is delta, then if the leading edge is supersonic,
        %span effects are pertinent
        if epsilon > mu   
            Clspan = (2*pi-4)*C1*alpha;
        else    
            Clspan = 0;
        end 
        %Combination of the lift and effects from span and profile.
        Cl = Cl0+Clprofile+Clspan;
    else
        %If the wing type is rectangular span effects are always pertinent
        %and cauclated differently.
        Clspan = -(4*C1*alpha/(2*AR*beta));
        %Combination of the lift and effects from span and profile.
        Cl = Cl0+Clprofile+Clspan;
    end
end

