function f = Shuttle_deriv(tau,X,U,t0,tf,p)
%--------------------------------------------------------------------------
% Shuttle_deriv(tau,X,U,t0,tf,p)
% Prepares the dynamic constraints for the solver for the Shuttle model
%--------------------------------------------------------------------------
    %Radius of Earth
    rE = 6.356766e6;
    
    %Atmospheric coefficients
    g0 = 9.80665;
    P0 = 101325;
    T0 = 288.15;
    rho0 = 1.225;
    Rgas = 8.3144;
    Mair = 0.02896;
    Vsound = 303; 

    %Gravity model
    g = g0.*(rE./(rE+X(:,3))).^2;

    %Atmosphere model
    rho = rho0.*exp(-g0.*Mair.*X(:,3)./(Rgas.*T0));   

    %Calculate Mach and angle of attack
    Mach = X(:,1)/Vsound;
    alpha = wrapToPi(U(:,2));

    %Apply limits on Mach and angle of attack for aerodynamic data
    for i = 1:1:length(Mach)
        if Mach(i,1) > 10
            Mach(i,1) = 10;
        end
    end

    for i = 1:1:length(alpha)
        if alpha(i,1) > pi/2
            aoa(i,1) = alpha(i,1) - pi;
        elseif alpha(i,1) < -pi/2
            aoa(i,1) = pi + alpha(i,1);
        else
            aoa(i,1) = alpha(i,1);
        end
    end

    %Get aerodynamic data
    Cl = getShuttleCl(Mach,aoa)+getShuttleClDelta(Mach).*U(:,1);
    Cm = getShuttleCm(Mach,aoa)+getShuttleCmDelta(Mach).*U(:,2);
    Cd = getShuttleCd(Mach,aoa);

    %Vehicle geometry
    db = 5;
    lb = 40;
    Ss = 2*pi*db*lb;

    %Reference geometry
    lref = lb;
    Sref = Ss;

    %Mass and moment of inertia
    m = 99318;
    Iyy = 1/4*m*(db/2)^2+(1/12)*m*lb^2;

    %Aerodynamic forces
    D = Sref.*Cd.*X(:,1).^2.*rho;
    L = Sref.*Cl.*X(:,1).^2.*rho;
    M = Sref.*lref.*Cm.*X(:,1).^2.*rho;

    %Equations of motion
    vdot = -D./m+g.*sin(X(:,2));
    ydot = -L./(m.*X(:,1))-X(:,1).*cos(X(:,2))./(rE+X(:,3))+g.*cos(X(:,2))./X(:,1);
    hdot = -X(:,1).*sin(X(:,2));
    thetadot = X(:,1).*cos(X(:,2))./(rE+X(:,3));
    qdot = M./Iyy;
    alphadot = X(:,5)-ydot+thetadot;

    %Passing them in
    f(:,1) = vdot;
    f(:,2) = ydot;
    f(:,3) = hdot;
    f(:,4) = thetadot;
    f(:,5) = qdot;
    f(:,6) = alphadot;
end