function [tout,Xout] = dynamicsInt()
%--------------------------------------------------------------------------
% dynamicsInt()
% Propegates the dynamics of a re-entry scenario using ODE45
%--------------------------------------------------------------------------
    %Input relevant vehicle parameters
    rE = 6.356766e6;
    m = 237700;
    db = 5;
    lb = 40;
    Sb = pi/4*db^2;
    Ss = pi*db*lb;
    Iyy = 1/4*m*(db/2)^2+(1/12)*m*lb^2; 
    
    %Input starting condition    
    [i,om_u,V0,y0,alt0,theta0,q0,alpha0] = startState();
    ini_val = [V0;y0;alt0;theta0;m0];         

    %Input time of flight for optimal trajectory        
    tstart = 0;
    tfinal = 400;  

    %Prepare ODE solver            
    options = odeset('Events',@events,'AbsTol',1e-14,'RelTol',1e-12);
    tout = tstart;
    Xout = ini_val.';
    altitudeTest = 1;
    tstep = 1;

    %ODE stops when vehicle hits ground
    while altitudeTest > 0 && tstart+tstep < tfinal
        % Solve until the first terminal event.
       [t,X] = ode45(@(t,X)f(t,X,m,Iyy,Ss,Sb,lb),tstart:tstep:tfinal,ini_val,options);

       % Accumulate output
       nt = length(t);
       tout = [tout; t(2:nt)];
       Xout = [Xout; X(2:nt,:)];
       V0 = Xout(end,1);
       y0 = Xout(end,2);
       alt0 = Xout(end,3);
       
       % Set the new initial conditions
       ini_val(:) = X(nt,:);
       tstart = t(nt);
       altitudeTest = X(end,3);
       counter = counter+1;
    end
end

%This function handles the vehicle dynamics to be loaded into ODE45                     
function Xdot = f(t,X,m,Iyy,Ss,Sb,lb)
    %Load parameters
    rE = 6.356766e6;
    Isp = 376; 
    Vsound = 303; 
    Pex = 36800;

    g0 = 9.80665;
    P0 = 101325;
    T0 = 288.15;
    rho0 = 1.225;

    Rgas = 8.3144;
    Mair = 0.02896;
    
    %Earth, atmosphere model    
    g = g0*(rE/(rE+X(3)))^2;
    rho = rho0*exp(-g0*Mair*X(3)/(Rgas*T0));  
    
    %Find Mach, angle of attack        
    Mach = X(1)/Vsound;
    alpha = wrapToPi(X(6));
    
    %Limit to bounds of aerodynamic data
    if Mach > 10
        Mach = 10;
    end

    if alpha > pi/2
        aoa = alpha - pi;
    elseif alpha < -pi/2
        aoa = pi + alpha;
    else
        aoa = alpha;
    end
    
    %Get aerodynamic data    
    Cd = getArianeCd(Mach,aoa);
    Cm = getArianeCd(Mach,aoa);    
    Cl = getArianeCl(Mach,aoa);

    %Vehicle model            
    db = 5;
    lb = 40;
    Xcg = 20;
    Ss = 2*pi*db*lb;
    
    %Reference geometries            
    lref = lb;
    Sref = Ss;
    m = 237700;
    Iyy = 1/4*m*(db/2)^2+(1/12)*m*lb^2;

    %Aerodynamic forces    
    D = Sref*Cd*X(1)^2*rho;
    L = Sref*Cl*X(1)^2*rho;
    M = Sref*lref*Cm*X(1)^2*rho;
    
    %Equations of motion        
    vdot = -D/m+g*sin(X(2));
    ydot = -L/(m*X(1))-X(1)*cos(X(2))/(rE+X(3))+g*cos(X(2))/X(1);
    hdot = -X(1)*sin(X(2));
    thetadot = X(1)*cos(X(2))/(rE+X(3));
    qdot = M/Iyy;
    alphadot = X(5)-ydot+thetadot;

    %Load into output        
    Xdot = zeros(size(X));
    Xdot(1) = vdot;
    Xdot(2) = ydot;
    Xdot(3) = hdot;
    Xdot(4) = thetadot; 
    Xdot(5) = qdot; 
    Xdot(6) = alphadot; 
    
end

%This stops the ODE45 when the vehicle altitude becomes zero
function [value,isterminal,direction] = events(t,X)
value = X(3);   
direction = 0;   
isterminal = 1;    
end